import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;

/** DemoCustomButtonUI2 - similar to <code>DemoCustomButtonUI</code>
* except defining buttons with
* rounded corners.<p>
*
* Additionally, this program includes hit detection code to
* appropriately determine when the mouse pointer is over top of the
* button.  The default behaviour of <code>ButtonUI</code> does not
* work because of the button's rounded corners.
* Consult the source code for further details.<p>
*
* Screen snaps:<p>
*
* <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Launch...</code><br>
* <center><img src="DemoCustomButtonUI2-1.gif"></center><p>
* 
* <code>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Custom button pressed...</code><br>
* <center><img src="DemoCustomButtonUI2-2.gif"></center><p>
*
* This program includes an improvement suggested by Dmitri Shuralyov (10-Feb-2009)
* <p>
* 
* @see <a href="DemoCustomButtonUI2.java">source code</a>
* @author Scott MacKenzie, 2007-2011
*/
public class DemoCustomButtonUI2
{
   public static void main(String[] args)
   {
      // use look and feel for my system (Win32)
      try {
         UIManager.setLookAndFeel(
            UIManager.getSystemLookAndFeelClassName());
      } catch (Exception e) {}

      DemoCustomButtonUI2Frame frame = new DemoCustomButtonUI2Frame();
      frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      frame.setTitle("DemoCustomButtonUI2");
      frame.pack();

      // position application window in the center of the display
      Dimension d1 = frame.getSize();
      Dimension d2 = Toolkit.getDefaultToolkit().getScreenSize();
      int x = (d2.width - d1.width) / 2;
      int y = (d2.height - d1.height) / 2; 
      frame.setLocation(new Point(x, y));

      frame.setVisible(true);
   }
}

class DemoCustomButtonUI2Frame extends JFrame implements ActionListener
{
    // the following avoids a "warning" with Java 1.5.0 complier (?)
	static final long serialVersionUID = 42L;

   JButton b1;
   JButton b2;

   public DemoCustomButtonUI2Frame()
   {
      final Color BLUE = new Color(47, 79, 136);

      // ----------------------------------
      // construct and configure components
      // ----------------------------------

      b1 = new JButton("Regular Button");
      b2 = new JButton("Custom Button");

      // set a custom UI delegate

      b2.setUI(new CustomButtonUI2());

      // -------------
      // add listeners
      // -------------

      b1.addActionListener(this);
      b2.addActionListener(this);

      // -----------------
      // layout components
      // -----------------

      JPanel panel = new JPanel();
      panel.add(b1);
      panel.add(b2);
      panel.setBorder(BorderFactory.createEmptyBorder(25, 25, 25, 25));
      panel.setBackground(BLUE);

      // make panel this JFrame's content pane

      this.setContentPane(panel);
   }

   // -------------------------------
   // implement ActionListener method
   // -------------------------------

   public void actionPerformed(ActionEvent ae)
   {
      if (ae.getSource() == b1)
         System.out.println("Regular button pressed");
      else if (ae.getSource() == b2)
         System.out.println("Custom button pressed");
   }

   // -----------
   // inner class
   // -----------

   class CustomButtonUI2 extends ButtonUI
   {
      final Insets I = new Insets(4, 16, 4, 16);

      public void installUI(JComponent c)
      {
         AbstractButton b = (AbstractButton)c;
         BasicButtonListener listener = new BasicButtonListener(b);
         b.addMouseListener(listener);
			b.setCursor(new Cursor(Cursor.HAND_CURSOR));
      }

      public void uninstallUI(JComponent c)
      {
         AbstractButton b = (AbstractButton)c;
         BasicButtonListener listener = new BasicButtonListener(b);
         b.removeMouseListener(listener);
      }

      public Insets getDefaultMargin(AbstractButton ab)
      {
         return I;
      }

      public Insets getInsets(JComponent c)
      {
         return I;
      }

      public Dimension getMaximumSize(JComponent c)
      {
         return this.getPreferredSize(c);
      }

      public Dimension getMinimumSize(JComponent c)
      {
         return this.getPreferredSize(c);
      }

      public Dimension getPreferredSize(JComponent c)
      {
         Graphics g = c.getGraphics();
         FontMetrics fm = g.getFontMetrics();

         Dimension d = new Dimension();
         d.width = fm.stringWidth(((JButton)c).getText()) + I.left + I.right;
         d.height = fm.getHeight() + I.top + I.bottom;
         return d;
      }

      public void paint(Graphics g, JComponent c)
      {
         Graphics2D g2 = (Graphics2D)g;

         AbstractButton b = (AbstractButton)c;
         ButtonModel bm = b.getModel();

         FontMetrics fm = g2.getFontMetrics();

         // define some colors for drawing
     
         final Color BLUE = new Color(47, 79, 136);
         final Color BLUE_LIGHT = new Color(107, 132, 181);
         final Color BLUE_DARK = new Color(32, 54, 94);
         final Color BORDER = new Color(152, 171, 206);

         Color backgroundColor;
         Color lineColor = BORDER; 

         //if (bm.isArmed() && !bm.isPressed())
         if (bm.isArmed())
         {
            backgroundColor = BLUE_LIGHT;
         }
         else if (bm.isPressed())
            backgroundColor = BLUE_DARK;
         else
            backgroundColor = BLUE;  // normal!

         // use a round rectangle for drawing

         Dimension d = c.getPreferredSize();
         int x = d.width - 1;    // x-coordinate of right edge
         int y = d.height - 1;   // y-coordinate of bottom edge
         final int LW = 2;       // line width
         int arc = y;

         g2.setColor(BLUE);
         g2.fillRect(0, 0, x + 1, y + 1);

         g2.setColor(lineColor);
         g2.fillRoundRect(0, 0, x, y, arc, arc);

         g2.setColor(backgroundColor);
         g2.fillRoundRect(LW, LW, x - 2 * LW, y - 2 * LW, arc, arc);

         // paint foreground

         g2.setColor(Color.white);
         String s = ((JButton)c).getText();
         x = I.left;
         y = I.top + fm.getAscent();
         Font f = fm.getFont();            
         //g2.setFont(new Font(f.getName(), Font.BOLD, f.getSize())); 
         g2.drawString(s, x, y);
      }

      /** Returns true if x,y is within the rounded rectangle representing
      * this button.
      */
      public boolean contains(JComponent c, int x, int y)
      {
         int w = c.getWidth();
         int h = c.getHeight();
         int arc = h;

         RoundRectangle2D.Double r =
            new RoundRectangle2D.Double(0, 0, w, h, arc, arc);

         return r.contains(x, y);
      }
   } 
}
